/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <math.h>
#include <string.h>
#include "mex.h"
#include "frame.h"
#include "stream.h"
#include "layer3.h"

#define INPUT_BUFFER_SIZE	(5*8192)
#define MP3_GRANULARITY 4600

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {

	/* Declarations */
	char *filename;
	int filename_len;
	FILE *mp3;
	struct mad_stream Stream;
	struct mad_frame Frame;
	unsigned char InputBuffer[INPUT_BUFFER_SIZE+MAD_BUFFER_GUARD];
	int gr,i;
	unsigned int frameNum;
	double *data = NULL;
	double *data2 = NULL;
	int data_size,data_size2;
	unsigned int stop;
	unsigned int channel;

	/* Check number of arguments */
	if (nrhs<1) mexErrMsgTxt( "Not enough input arguments.");
	if (nlhs>2) mexErrMsgTxt( "Too many output arguments.");

	/* Input */
	filename_len = mxGetN(prhs[0])+1;
	filename = (char*)mxCalloc(filename_len, sizeof(char));
	mxGetString(prhs[0],filename,filename_len);
	mp3 = fopen(filename,"r");
	if (mp3==NULL) mexErrMsgTxt( "Cannot open file\n");
	channel = 0;
	if (nrhs>=2) channel = (unsigned int)mxGetPr(prhs[1])[0];
	if (channel>1) mexErrMsgTxt( "Channel parameter must be 0 or 1.");
	stop = 3;
	if (nrhs>=3) stop = (unsigned int)mxGetPr(prhs[2])[0];
	if (stop>5||stop<1) mexErrMsgTxt( "Stop parameter must be in the range [1,5].\n");

	/* Decoding */
	mad_stream_init(&Stream);
	mad_frame_init(&Frame);
	frameNum = 0;
	if (nlhs>0) {
		data_size = 1152*MP3_GRANULARITY;
		data = (double*) mxCalloc(data_size,sizeof(double));
	}
	if (nlhs>1) {
		data_size2 = 2*MP3_GRANULARITY;
		data2 = (double*) mxCalloc(data_size2,sizeof(double));
	}
	do {
		fail:
		if(Stream.buffer==NULL || Stream.error==MAD_ERROR_BUFLEN) {
			size_t ReadSize,Remaining;
			unsigned char *ReadStart;
			if(Stream.next_frame!=NULL) {
				Remaining=Stream.bufend-Stream.next_frame;
				memmove(InputBuffer,Stream.next_frame,Remaining);
				ReadStart=InputBuffer+Remaining;
				ReadSize=INPUT_BUFFER_SIZE-Remaining;
			} else { 
				ReadSize=INPUT_BUFFER_SIZE,
				ReadStart=InputBuffer,
				Remaining=0;
			}
			ReadSize=fread(ReadStart,1,ReadSize,mp3);
			if(ReadSize<=0) break;
			mad_stream_buffer(&Stream,InputBuffer,ReadSize+Remaining);
			Stream.error = MAD_ERROR_NONE;
		}
		Frame.options = Stream.options;
		if (!(Frame.header.flags & MAD_FLAG_INCOMPLETE) && mad_header_decode(&Frame.header, &Stream) == -1) goto fail;

		if (frameNum==0) {
			if (Frame.header.layer!=3) mexErrMsgTxt( "File is not mp3.\n");
			if (Frame.header.mode==0) channel = 0;
		}

		Frame.header.flags &= ~MAD_FLAG_INCOMPLETE;
		mad_layer_III(&Stream, &Frame, stop);

		if (nlhs>0) {
			if ((frameNum*2+2)*576>=data_size) {
				data_size += 1152*MP3_GRANULARITY;
				data = (double*) mxRealloc(data, data_size*sizeof(double));
			}
			for (gr=0;gr<2;gr++)
				for (i=0;i<576;i++)
					data[(frameNum*2+gr)*576+i] = Frame.xr[gr][channel][i];
		}
		if (nlhs>1) {
			if (frameNum*2+2>=data_size2) {
				data_size2 += 2*MP3_GRANULARITY;
				data2 = (double*) mxRealloc(data2, data_size2*sizeof(double));
			}
			for (gr=0;gr<2;gr++)
				data2[frameNum*2+gr] = Frame.block_type[gr][channel];
		}
		frameNum++;
	}while(1);

	/* End decoding */
	mad_frame_finish(&Frame);
	mad_stream_finish(&Stream);

	/* Output */
	if (nlhs>0) {
		plhs[0] = mxCreateDoubleMatrix(576,frameNum*2,mxREAL);
		memcpy(mxGetPr(plhs[0]),data,sizeof(double)*frameNum*2*576);
	}
	if (nlhs>1) {
		plhs[1] = mxCreateDoubleMatrix(1,frameNum*2,mxREAL);
		memcpy(mxGetPr(plhs[1]),data2,sizeof(double)*frameNum*2);
	}

	/* Clean */
	if (data) mxFree( data );
	if (data2) mxFree( data2 );
	fclose( mp3 );
	mxFree( filename );
}

