clear all
close all

disp(' ');
disp('Compute 13 transform-domain MFCC using mp3mfcc.');
mfccL = mp3mfcc('dylan.mp3',13,0);
mfccR = mp3mfcc('dylan.mp3',13,1);

disp('Read MDCT coefficients using mp3mdct.');
cL = mp3mdct('dylan.mp3',0);
cR = mp3mdct('dylan.mp3',1);

disp('Init triangular filterbank.');
low = 20;
high = 16000;
N = 1152;
M = 40;
Fs = 44100;
lowestMel = log(1.0+low/700.0)*1127.01048;
highestMel = log(1.0+high/700.0)*1127.01048;
freqs = zeros(1,M+2);
for n=1:M+2
    freqs(n) = 700.0*( exp( (lowestMel+((n-1)/(M+1))*(highestMel-lowestMel)) / 1127.01048 ) - 1 );
end
lower = freqs(1:end-2);
center = freqs(2:end-1);
upper = freqs(3:end);
heights = 2./(upper-lower);
fftfs = ((0:N/2-1)+0.5)*44100/N;
filters = zeros(M,N/2);
for n=1:M
    for m=1:N/2
        if fftfs(m)>lower(n)&&fftfs(m)<=center(n)
            filters(n,m) = heights(n) * (fftfs(m)-lower(n))/(center(n)-lower(n));
        end
        if fftfs(m)>center(n)&&fftfs(m)<=upper(n)
            filters(n,m) = heights(n) * (upper(n)-fftfs(m))/(upper(n)-center(n));
        end
    end
end

disp('Apply triangular filterbank on MDCT coefficients.');
fboutL = filters * abs(cL);
fboutR = filters * abs(cR);

disp('Take Log10.');
fboutL = log10( fboutL + 1e-16 );
fboutR = log10( fboutR + 1e-16 );

disp('Init DCT.');
DCT = 1/sqrt(M/2)*cos((0:(M-1))' * (2*(0:(M-1))+1) * pi/2/M);
DCT(1,:) = DCT(1,:) * sqrt(2)/2;

disp('Apply DCT on filterbank output.');
mfcc2L = DCT * fboutL;
mfcc2R = DCT * fboutR;

disp('Keep 13th first coefficients.');
mfcc2L = mfcc2L(1:13,:);
mfcc2R = mfcc2R(1:13,:);

eL = sum(sum((mfccL-mfcc2L).^2));
eR = sum(sum((mfccR-mfcc2R).^2));
disp(['Error between the two implementation. Left: ' num2str(eL) ', Right: ' num2str(eR) '.']);
disp(' ');